# -*- coding: utf-8 -*-

from django.db import migrations


def populate_videos(apps, schema_editor):
    Match = apps.get_model("competition", "Match")
    for match in Match.objects.all():
        match.videos = [video.url for video in match.matchvideo_set.all()]
        match.save()


def noop(apps, schema_editor):
    pass


class Migration(migrations.Migration):

    dependencies = [
        ("competition", "0026_matchvideo"),
    ]

    operations = [
        migrations.RunPython(populate_videos, noop),
    ]



from django.urls import reverse_lazy
from django.utils.functional import cached_property


class AdminUrlMixin(object):
    """
    Mixin which can be used to add API to a model for use by standard admin
    component templates, tags, and filters.
    """

    def _get_admin_namespace(self):
        raise NotImplementedError

    def _get_url_args(self):
        raise NotImplementedError

    @cached_property
    def urls(self):
        namespace = self._get_admin_namespace()
        args = self._get_url_args()
        crud = {
            "edit": reverse_lazy("%s:edit" % namespace, args=args),
            "delete": reverse_lazy("%s:delete" % namespace, args=args),
            "perms": reverse_lazy("%s:perms" % namespace, args=args),
        }
        return crud



from django.contrib.syndication.views import Feed
from django.utils.feedgenerator import Rss201rev2Feed

from touchtechnology.news.models import Article


class ExtendedRSSFeed(Rss201rev2Feed):
    """
    Create a type of RSS feed that has content:encoded elements.
    """

    content_type = "text/xml; charset=utf-8"

    def root_attributes(self):
        attrs = super().root_attributes()
        attrs["xmlns:content"] = "http://purl.org/rss/1.0/modules/content/"
        return attrs

    def add_item_elements(self, handler, item):
        super().add_item_elements(handler, item)
        handler.addQuickElement("content:encoded", item["content_encoded"])


class NewsFeed(Feed):
    def items(self):
        return Article.objects.live().prefetch_related("categories")

    def item_categories(self, item):
        return [c.title for c in item.categories.all()]

    def item_description(self, item):
        return item.abstract

    def item_link(self, item):
        return item.get_absolute_url()

    def item_pubdate(self, item):
        return item.published

    def item_extra_kwargs(self, item):
        return {
            "content_encoded": f"<blockquote>{item.abstract}</blockquote>\n{item.copy}"
        }



from dateutil.relativedelta import relativedelta
from django.shortcuts import get_object_or_404
from django.urls import path
from django.utils.feedgenerator import Atom1Feed

from touchtechnology.common.sites import Application
from touchtechnology.news.app_settings import PAGINATE_BY
from touchtechnology.news.decorators import (
    date_view,
    last_modified_article,
    news_last_modified,
)
from touchtechnology.news.models import Article, Category
from touchtechnology.news.syndication import ExtendedRSSFeed, NewsFeed

YEAR_DELTA = relativedelta(years=1)
MONTH_DELTA = relativedelta(months=1)
DAY_DELTA = relativedelta(days=1)


class NewsSite(Application):
    def __init__(self, name="news", app_name="news", **kwargs):
        self.node = kwargs.get("node")  # store the node for future reference
        super().__init__(name=name, app_name=app_name, **kwargs)

    def get_urls(self):
        return [
            path(
                "",
                self.index,
                name="index",
            ),
            path("<int:year>/", self.archive_year, name="year"),
            path(
                "<int:year>/<str:month>/",
                self.archive_month,
                name="month",
            ),
            path(
                "<int:year>/<str:month>/<int:day>/",
                self.archive_day,
                name="day",
            ),
            path(
                "<int:year>/<str:month>/<int:day>/<slug:slug>/",
                self.article,
                name="article",
            ),
            path(
                "<int:year>/<str:month>/<int:day>/<slug:slug>/<str:locale>/",
                self.translation,
                name="translation",
            ),
            path("feeds/", self.feeds, name="feeds"),
            path("feeds/atom/", self.feed, kwargs={"format": "atom"}, name="feed-atom"),
            path("feeds/rss/", self.feed, kwargs={"format": "rss"}, name="feed-rss"),
            path("archive/", self.archive, name="archive"),
            path("<str:category>/", self.list_articles, name="category"),
        ]

    @property
    def urls(self):
        return self.get_urls(), self.app_name, self.name

    @property
    def categories(self):
        return Category.objects.all()

    @news_last_modified
    def index(self, request, **extra_context):
        return self.generic_list(
            request,
            Article.objects.live()[:PAGINATE_BY],
            templates=self.template_path("index.html"),
            extra_context=extra_context,
        )

    @news_last_modified
    def list_articles(self, request, **extra_context):
        queryset = Article.objects.live()
        category = extra_context.get("category")
        if category is not None:
            queryset = queryset.filter(categories__slug=category)
        return self.generic_list(
            request,
            Article,
            queryset=queryset,
            extra_context=extra_context,
        )

    @date_view
    @last_modified_article
    def article(self, request, date, slug, **extra_context):
        return self.generic_detail(
            request,
            Article.objects.live(),
            published__range=(date, date + DAY_DELTA),
            slug=slug,
            extra_context=extra_context,
        )

    @date_view
    @last_modified_article
    def translation(self, request, date, slug, locale, **extra_context):
        article = get_object_or_404(
            Article.objects.live(),
            published__range=(date, date + DAY_DELTA),
            slug=slug,
        )
        return self.generic_detail(
            request,
            article.translations,
            locale=locale,
            extra_context=extra_context,
        )

    @date_view
    @last_modified_article
    def archive_day(self, request, date, **extra_context):
        extra_context["day"] = date
        return self.generic_list(
            request,
            Article.objects.live().filter(published__range=(date, date + DAY_DELTA)),
            templates=self.template_path("archive/day.html"),
            extra_context=extra_context,
        )

    @date_view
    @last_modified_article
    def archive_month(self, request, date, **extra_context):
        queryset = Article.objects.filter(
            published__range=(date, date + MONTH_DELTA)
        ).live()
        extra_context.update(
            {"date_list": queryset.dates("published", "day"), "month": date}
        )
        return self.generic_list(
            request,
            Article,
            queryset=queryset,
            templates=self.template_path("archive/month.html"),
            extra_context=extra_context,
        )

    @date_view
    @last_modified_article
    def archive_year(self, request, date, **extra_context):
        queryset = Article.objects.filter(
            published__range=(date, date + YEAR_DELTA)
        ).live()
        extra_context.update(
            {
                "date_list": queryset.dates("published", "month"),
                "year": date.year,
            }
        )
        return self.generic_list(
            request,
            Article,
            queryset=queryset,
            templates=self.template_path("archive/year.html"),
            extra_context=extra_context,
        )

    def archive(self, request, **extra_context):
        queryset = Article.objects.live()
        extra_context.update(
            {
                "date_list": queryset.dates("published", "year"),
                "latest": queryset.order_by("-published")[: (3 * PAGINATE_BY)],
            }
        )
        return self.generic_list(
            request,
            Article,
            queryset=queryset,
            templates=self.template_path("archive/index.html"),
            extra_context=extra_context,
        )

    def feeds(self, request, **kwargs):
        context = {
            "feeds": {
                "ATOM": "atom",
                "RSS": "rss",
            },
        }
        context.update(kwargs)
        templates = self.template_path("feeds.html")
        return self.render(request, templates, context)

    def feed(self, request, format, **kwargs):
        class BaseFeed(NewsFeed):
            title = getattr(request, "tenant", None) or kwargs.get("node")
            link = self.reverse("feeds")

        class AtomNewsFeed(BaseFeed):
            feed_type = Atom1Feed

        class RssNewsFeed(BaseFeed):
            feed_type = ExtendedRSSFeed

        feed_class = {
            "atom": AtomNewsFeed,
            "rss": RssNewsFeed,
        }
        return feed_class[format]()(request)


news = NewsSite()



# -*- coding: utf-8 -*-

import django.contrib.postgres.fields
import django.db.models.deletion
from django.db import migrations, models

import touchtechnology.common.db.models


class Migration(migrations.Migration):

    dependencies = [
        ("competition", "0025_matchvideo_baseline"),
    ]

    operations = [
        migrations.AddField(
            model_name="match",
            name="videos",
            field=django.contrib.postgres.fields.ArrayField(
                base_field=models.URLField(), null=True, size=None
            ),
        ),
        migrations.AlterField(
            model_name="matchvideo",
            name="match",
            field=touchtechnology.common.db.models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE, to="competition.Match"
            ),
        ),
    ]



"""
Originally copied from http://stackoverflow.com/a/13842223
"""

from django.db.models.fields.related import RelatedField
from django.forms.models import model_to_dict


class ModelDiffMixin(object):
    """
    A model mixin that tracks model fields' values and provide some useful api
    to know what fields have been changed.
    """

    def __init__(self, *args, **kwargs):
        super(ModelDiffMixin, self).__init__(*args, **kwargs)
        self.__initial = self._dict

    @property
    def diff(self):
        d1 = self.__initial
        d2 = self._dict
        diffs = [(k, (v, d2[k])) for k, v in d1.items() if v != d2[k]]
        return dict(diffs)

    @property
    def has_changed(self):
        return bool(self.diff)

    @property
    def changed_fields(self):
        return self.diff.keys()

    def get_field_diff(self, field_name):
        """
        Returns a diff for field if it's changed and None otherwise.
        """
        return self.diff.get(field_name, None)

    def save(self, *args, **kwargs):
        """
        Saves model and set initial state.
        """
        super(ModelDiffMixin, self).save(*args, **kwargs)
        self.__initial = self._dict

    @property
    def _dict(self):
        fields = [
            field.name
            for field in self._meta.get_fields()
            if not isinstance(field, RelatedField)
            and field.name not in self.get_deferred_fields()
        ]
        data = model_to_dict(self, fields=fields)
        return data



# -*- coding: utf-8 -*-

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("content", "0002_last_modified"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="placeholder",
            options={"ordering": ("path",)},
        ),
        migrations.AlterField(
            model_name="pagecontent",
            name="label",
            field=models.SlugField(
                choices=[("copy", "copy")],
                default="copy",
                max_length=20,
                verbose_name="CSS class",
            ),
        ),
    ]



# -*- coding: utf-8 -*-

from django.db import migrations

import touchtechnology.common.db.models


class Migration(migrations.Migration):
    dependencies = [
        ("competition", "0034_orderedsitemapnode_add_copy_field"),
    ]

    operations = [
        migrations.AddField(
            model_name="season",
            name="disable_calendar",
            field=touchtechnology.common.db.models.BooleanField(
                default=False,
                help_text=(
                    "Set to prevent the iCalendar feature for this season. "
                    "Will hide icon in front-end and disable functionality. "
                    "Batch process may disable after last match of tournament "
                    "has taken place."
                ),
            ),
        ),
    ]



# -*- coding: utf-8 -*-

from django.db import migrations, models

import touchtechnology.common.db.models


class Migration(migrations.Migration):

    dependencies = [
        ("common", "0005_sitemapnode_populate_kwargs"),
    ]

    operations = [
        migrations.AlterField(
            model_name="sitemapnode",
            name="enabled",
            field=touchtechnology.common.db.models.BooleanField(
                default=True,
                help_text="Set this to 'No' to disable this object and it's children on the site.",
                verbose_name="Enabled",
            ),
        ),
        migrations.AlterField(
            model_name="sitemapnode",
            name="hidden_from_navigation",
            field=touchtechnology.common.db.models.BooleanField(
                default=False,
                help_text="When set to 'Yes' this object will still be available, but will not appear in menus.",
                verbose_name="Hide from menus",
            ),
        ),
        migrations.AlterField(
            model_name="sitemapnode",
            name="hidden_from_robots",
            field=touchtechnology.common.db.models.BooleanField(
                default=False,
                help_text="Set this to <em>Yes</em> to prevent search engines from indexing this part of the site.<br><strong>Warning:</strong> this may affect your ranking in search engines.",
                verbose_name="Hide from spiders",
            ),
        ),
        migrations.AlterField(
            model_name="sitemapnode",
            name="hidden_from_sitemap",
            field=touchtechnology.common.db.models.BooleanField(
                default=False,
                help_text="When set to 'Yes' this object will not be listed in the auto-generated sitemap.",
                verbose_name="Hide from sitemap",
            ),
        ),
        migrations.AlterField(
            model_name="sitemapnode",
            name="last_modified",
            field=models.DateTimeField(auto_now=True),
        ),
        migrations.AlterField(
            model_name="sitemapnode",
            name="require_https",
            field=touchtechnology.common.db.models.BooleanField(
                default=False,
                help_text="Force this to be served via HTTPS.",
                verbose_name="HTTPS Required",
            ),
        ),
        migrations.AlterField(
            model_name="sitemapnode",
            name="restrict_to_groups",
            field=models.ManyToManyField(
                blank=True,
                help_text="If you select one or more of these groups your visitors will need to be logged in and a member of an appropriate group to view this part of the site.",
                to="auth.Group",
                verbose_name="Restrict to Groups",
            ),
        ),
        migrations.AlterField(
            model_name="sitemapnode",
            name="slug_locked",
            field=touchtechnology.common.db.models.BooleanField(
                default=False, verbose_name="Slug locked"
            ),
        ),
    ]



from django import forms


class URLInput(forms.URLInput):
    def __init__(self, attrs=None):
        if attrs is None:
            attrs = {"class": "form-control"}
        super(URLInput, self).__init__(attrs)


class URLField(forms.URLField):
    widget = URLInput



from unittest import mock

from test_plus import TestCase

from touchtechnology.common.forms.fields import boolean_choice_field_coerce
from touchtechnology.common.forms.tz import timezone_choice
from touchtechnology.common.utils import get_base_url, get_mod_func


class BooleanChoiceFieldCoerce(TestCase):
    def test_true_boolean(self):
        result = boolean_choice_field_coerce(True)
        self.assertTrue(result)

    def test_false_boolean(self):
        result = boolean_choice_field_coerce(False)
        self.assertFalse(result)

    def test_true_integer(self):
        result = boolean_choice_field_coerce("1")
        self.assertTrue(result)

    def test_false_integer(self):
        result = boolean_choice_field_coerce("0")
        self.assertFalse(result)

    def test_true_string(self):
        result = boolean_choice_field_coerce(1)
        self.assertTrue(result)

    def test_false_string(self):
        result = boolean_choice_field_coerce(0)
        self.assertFalse(result)


class GetModFunc(TestCase):

    def test_get_mod_func(self):
        mod_name, func_name = get_mod_func(
            "django.contrib.auth.tokens.default_token_generator"
        )
        self.assertEqual(mod_name, "django.contrib.auth.tokens")
        self.assertEqual(func_name, "default_token_generator")

    def test_no_dot(self):
        mod_name, func_name = get_mod_func("django")
        self.assertEqual(mod_name, "django")
        self.assertEqual(func_name, "")


class TimeZoneChoice(TestCase):
    def test_one_part(self):
        self.assertEqual(timezone_choice("UTC"), ("UTC", "UTC"))

    def test_two_parts(self):
        self.assertEqual(
            timezone_choice("Australia/Sydney"), ("Australia/Sydney", "Sydney")
        )

    def test_three_parts(self):
        self.assertEqual(
            timezone_choice("America/Indiana/Indianapolis"),
            ("America/Indiana/Indianapolis", "Indianapolis (Indiana)"),
        )


class GetBaseUrl(TestCase):

    @mock.patch("touchtechnology.common.utils.connection")
    def test_tenant_no_prepend_www(self, connection):
        connection.tenant = mock.Mock(["domain_url"])
        connection.tenant.domain_url = "example.com"
        self.assertEqual("http://example.com/", get_base_url())

    @mock.patch("touchtechnology.common.utils.connection")
    def test_tenant_prepend_www_true(self, connection):
        connection.tenant = mock.Mock(["domain_url", "prepend_www"])
        connection.tenant.domain_url = "example.com"
        connection.tenant.prepend_www = True
        self.assertEqual("http://www.example.com/", get_base_url())

    @mock.patch("touchtechnology.common.utils.connection")
    def test_tenant_prepend_www_false(self, connection):
        connection.tenant = mock.Mock(["domain_url", "prepend_www"])
        connection.tenant.domain_url = "example.com"
        connection.tenant.prepend_www = False
        self.assertEqual("http://example.com/", get_base_url())

    def test_site(self):
        self.assertEqual("http://example.com/", get_base_url())



import factory
from django.contrib.auth.models import User
from django.utils import timezone
from factory.django import DjangoModelFactory

from touchtechnology.common.models import SitemapNode


class SitemapNodeFactory(DjangoModelFactory):
    class Meta:
        model = SitemapNode

    title = factory.Faker("country")


class UserFactory(DjangoModelFactory):
    class Meta:
        model = User
        django_get_or_create = ("username",)

    username = factory.Sequence(lambda n: "username{0}".format(n + 1))

    first_name = factory.Faker("first_name")
    last_name = factory.Faker("last_name")

    email = factory.LazyAttribute(
        lambda a: "{0}@example.com".format(a.first_name.lower())
    )
    date_joined = factory.LazyFunction(timezone.now)

    password = factory.PostGenerationMethodCall("set_password", "password")



import factory
from factory.django import DjangoModelFactory

from touchtechnology.content.models import Redirect


class RedirectFactory(DjangoModelFactory):

    class Meta:
        model = Redirect

    source_url = factory.Faker("uri_path")
    destination_url = factory.Faker("uri_path")



import logging
from functools import wraps

logger = logging.getLogger(__name__)


def disable_for_loaddata(signal_handler):
    """
    Decorator that turns off signal handlers when loading
    fixture data.
    """

    @wraps(signal_handler)
    def wrapper(*args, **kwargs):
        raw = kwargs.get("raw", False)
        if raw:
            instance = kwargs.get("instance")
            logger.debug(
                "Skipping signal handler %r for instance %r.", signal_handler, instance
            )
            return
        signal_handler(*args, **kwargs)

    return wrapper



import logging

logger = logging.getLogger(__name__)

NAME = "Common Components"
INSTALL = ("AccountsSite",)

__version__ = "3.5.5"

logger.debug('"%s"/"%s"' % (NAME, __version__))

default_app_config = "touchtechnology.common.apps.CommonConfig"



from django.conf import settings

__all__ = (
    "APP_ROUTING",
    "CURRENCY_ABBREVIATION",
    "CURRENCY_SYMBOL",
    "PAGINATE_BY",
    "SITEMAP_CACHE_DURATION",
    "SITEMAP_EDIT_PARENT",
    "SITEMAP_HTTPS_OPTION",
    "SITEMAP_ROOT",
    "STORAGE_FOLDER",
    "STORAGE_URL",
)


def A(n, d):
    return getattr(settings, "AUTHENTICATION_" + n, d)


def S(n, d=None):
    return getattr(settings, "TOUCHTECHNOLOGY_" + n, d)


APP_ROUTING = S("APP_ROUTING", ())
CURRENCY_ABBREVIATION = S("CURRENCY_ABBREVIATION", "AUD")
CURRENCY_SYMBOL = S("CURRENCY_SYMBOL", "$")
PAGINATE_BY = S("PAGINATE_BY", 5)
PROFILE_FORM_CLASS = S(
    "PROFILE_FORM_CLASS", "touchtechnology.common.forms_lazy.ProfileForm"
)
SITEMAP_CACHE_DURATION = S("SITEMAP_DURATION")
SITEMAP_EDIT_PARENT = S("SITEMAP_EDIT_PARENT", False)
SITEMAP_HTTPS_OPTION = S("HTTPS_OPTION", False)
SITEMAP_ROOT = S("SITEMAP_ROOT")
STORAGE_FOLDER = S("STORAGE_FOLDER")
STORAGE_URL = S("STORAGE_URL")



# -*- coding: utf-8 -*-

import django.db.models.deletion
from django.db import migrations

import touchtechnology.common.db.models


class Migration(migrations.Migration):

    dependencies = [
        ("competition", "0013_statistic_played_field"),
    ]

    operations = [
        migrations.AlterField(
            model_name="match",
            name="play_at",
            field=touchtechnology.common.db.models.ForeignKey(
                on_delete=django.db.models.deletion.SET_NULL,
                blank=True,
                to="competition.Place",
                null=True,
            ),
        ),
    ]



# -*- coding: utf-8 -*-

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("competition", "0015_person_user_relation"),
    ]

    operations = [
        migrations.AddField(
            model_name="season",
            name="forfeit_notifications",
            field=models.ManyToManyField(
                help_text="When a team advises they are forfeiting, notify the opposition team plus these people.",
                to=settings.AUTH_USER_MODEL,
                blank=True,
            ),
        ),
    ]



import datetime

import factory
import factory.fuzzy
from django.utils.text import slugify
from factory.django import DjangoModelFactory

from touchtechnology.news.models import Article, Category, Translation


class ArticleFactory(DjangoModelFactory):
    class Meta:
        model = Article

    headline = factory.Faker("sentence")
    abstract = factory.Faker("paragraph")
    published = factory.fuzzy.FuzzyDateTime(
        datetime.datetime(2013, 7, 15, tzinfo=datetime.timezone.utc),
        force_microsecond=0,
    )
    copy = factory.Faker("paragraph")
    slug = factory.LazyAttribute(lambda a: slugify(a.headline))


class TranslationFactory(DjangoModelFactory):
    class Meta:
        model = Translation

    headline = factory.Faker("sentence")
    abstract = factory.Faker("paragraph")
    copy = factory.Faker("paragraph")


class CategoryFactory(DjangoModelFactory):
    class Meta:
        model = Category

    title = factory.Faker("country")



from test_plus import TestCase

from touchtechnology.common.tests.factories import UserFactory


class AuthenticationTest(TestCase):

    def test_admin_login(self):
        self.get("admin:index")
        self.response_302()

    def test_admin_logout(self):
        user = UserFactory.create(is_staff=True)
        with self.login(user):
            # In Django 5.0 the logout view was changed to not allow GET requests.
            self.post("accounts:logout")
            self.assertResponseContains("<p>You have successfully been logged out.</p>")



# -*- coding: utf-8 -*-

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("competition", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="division",
            name="draft",
            field=models.BooleanField(
                default=True,
                help_text="Marking a division as draft will prevent matches from being visible in the front-end.",
            ),
            preserve_default=True,
        ),
    ]
